#!/bin/sh
# Update awstats database. 
#
# Doug Teodoro, Barry Blumenfeld, Lee Lueking. 4 Dec. 2006
# Modified by Dave Dykstra September 2007
# Modified by Barry Blumenfeld October 2007 to squid log reformatting and compression
# Modified by David Front 2010

AWSTATS_LOGNAME=$1

AWSTATS_ETC_DIR_=/etc/awstats
AWSTATS_DATA_DIR_BASE=/var/lib/frontier-awstats
AWSTATSSCRIPTS_DIR=/usr/share/frontier-awstats

PASSWORD_FILE=$AWSTATS_ETC_DIR_/password-file

if [ ! -s ${PASSWORD_FILE} ]; then
	echo "ERROR: password is missing: ${PASSWORD_FILE}. Leaving"
	# TBD: Take care that an error email will be sent to administrators
    exit 1
fi

AWSTATSCONF_FILE=$AWSTATS_ETC_DIR_/awstatsconf
if [ -f $AWSTATSCONF_FILE ]; then
    source $AWSTATSCONF_FILE 
fi

function getKeyFromFileWithDelimiter(){
    # $1 key
    # $2 file
    # $3 delimiter
    #echo "grep "^${1}" $2|awk -F $3 '{print $2}'"
    value=`grep "^${1}" $2|awk -F "$3" '{print $2}'`
    if [  X$value == X ]; then
        echo "ERROR: Did not find key: $1, at file: $2, with delimiter: $3"
        exit 1
    fi
    #echo "file: $1, key: $2, delimiter: $3, value: $value"
    echo $value
}

function condexport {
    eval "if [ ! \"\${${1}+x}\" ] ; then
       export ${1}=${2}
    fi"
}

function findNodeAndProject {
	mappingName=awstatsSiteProjectNodesMapping
	mappingFile=/tmp/${mappingName}
	/bin/rm -f ${mappingFile} ${mappingFile}_
	wget -O ${mappingFile}_ http://wlcg-squid-monitor.cern.ch/${mappingName}
	grep -v '#' ${mappingFile}_ | grep `dnsdomainname` > ${mappingFile}
	nodes=`awk '{print $2}' ${mappingFile}`
	#echo "nodes: ${nodes}"
	FOUND=NO
	THISNODE=`uname -n`
	for NODE in ${nodes} ; do
		if [ "$NODE" = "$THISNODE" ] ; then
			FOUND=YES
		else
			case `host $NODE 2>/dev/null` in
		    *"is an alias for $THISNODE"*)
			FOUND=YES
			;;
			esac
	    fi
	    if [ $FOUND == YES ] ; then
			AWSTATS_CONFIG=`grep ${NODE} ${mappingFile}| head -1| awk '{print $3}'`
			SITEPROJ=`grep ${NODE} ${mappingFile}| head -1| awk '{print $1}'`
			break
	    fi
	done
	/bin/rm -f ${mappingFile} ${mappingFile}_
	if [ ! $FOUND == YES ]; then
	    if [ -n "$AWSTATS_NODENAME" ]; then
		echo "WARNING: this node was not found at the list of nodes that report to awstats"
		echo "Using the saved value"
		AWSTATS_CONFIG=$AWSTATS_NODENAME
		SITEPROJ=$AWSTATS_SITEPROJ
	    else
		echo "ERROR: this node was not found at the list of nodes that report to awstats"
		exit 1
	    fi
	elif [ "$AWSTATS_NODENAME" != "$AWSTATS_CONFIG" ] || \
		[ "$AWSTATS_SITEPROJ" != "$SITEPROJ" ]; then
	    rm -f $AWSTATSCONF_FILE
	    (echo "# Automatically generated by $0 on `date`"
	    echo "export AWSTATS_NODENAME=$AWSTATS_CONFIG"
	    echo "export AWSTATS_SITEPROJ=$SITEPROJ") >$AWSTATSCONF_FILE
	fi
}
findNodeAndProject

# Find squid number if this is for a multisquid setup, that is, the 
#  access log name ends in /squidNN.
# In order to preserve data when switching between non-multisquid and
#  multisquid, treat the first one (/squid0) almost exactly like a
#  non-multisquid setup except for a -0 appended to the upload directory
#  name, and for the others add -NN to the configuration too
AWSTATS_UPLOAD_DIR="$AWSTATS_CONFIG"
case $AWSTATS_LOGNAME in
    */squid[0-9]/access*|*squid[1-9][0-9]/access*)
	SQUIDNUM="`echo "$AWSTATS_LOGNAME"|sed 's,.*/squid,,;s,/.*,,'`"
	if [ "$SQUIDNUM" != 0 ]; then
	    AWSTATS_CONFIG="$AWSTATS_CONFIG-$SQUIDNUM"
	fi
	AWSTATS_UPLOAD_DIR="$AWSTATS_UPLOAD_DIR-$SQUIDNUM"
	;;
esac

CONF_FILE=$AWSTATS_ETC_DIR_/awstats.$AWSTATS_CONFIG.conf

AWSTATS_PL=$AWSTATSSCRIPTS_DIR/awstats.pl

echo `date`

echo "Calculating awstats for $SITEPROJ-$AWSTATS_CONFIG"

# Handle awstats configuration:
# AWSTATS_DATA_DIR should be a unique directory for awstats data, otherwise, this script may rm by mistake needed files
AWSTATS_DATA_DIR=${AWSTATS_DATA_DIR_BASE}/$AWSTATS_CONFIG
#echo "AWSTATS_DATA_DIR_BASE: $AWSTATS_DATA_DIR_BASE, AWSTATS_DATA_DIR $AWSTATS_DATA_DIR"
sed 's,^DirData.*,DirData='${AWSTATS_DATA_DIR}',' ${AWSTATS_ETC_DIR_}/awstats.model.conf >${CONF_FILE}.tmp
if cmp -s ${CONF_FILE}.tmp ${CONF_FILE}; then
    # no change, no need to update
    /bin/rm -f ${CONF_FILE}.tmp
else
    /bin/mv -f ${CONF_FILE}.tmp ${CONF_FILE}
fi

if [ ! -d $AWSTATS_DATA_DIR ]; then
      mkdir -p $AWSTATS_DATA_DIR
fi

#  Change the URL into just the servlet name and the following component,
#  because awstats can't ignore the portion after the servlet name when the
#  delimiter is a '/' instead of question mark, as is the case starting in
#  frontier_client 2.8.0.  Include the component following the servlet name
#  because that's important information for CVMFS (it's always just "Frontier"
#  for Frontier).
TMPFILE=$AWSTATS_LOGNAME.shorturl
rm -f $TMPFILE
awk '{sub("http://[^/]*/","",$7);split($7,a,"/");$7=a[1] "/" a[2];print}' $AWSTATS_LOGNAME >$TMPFILE

#  Monthly stats
${AWSTATS_PL} -update -LogFile=${TMPFILE} -config=$AWSTATS_CONFIG 
#  Daily stats
${AWSTATS_PL} -update -LogFile=${TMPFILE} -config=$AWSTATS_CONFIG -databasebreak=day
rm -f ${TMPFILE}
#  Delete stats files older than 20 days, keeping the others in case transfers are down for a short while
find $AWSTATS_DATA_DIR -mtime +20 | xargs -t rm -f

#  Transfer to squid monitoring server for publishing
rsync --timeout=30 --contimeout=10 --password-file=$PASSWORD_FILE -av $AWSTATS_DATA_DIR/ $SITEPROJ@wlcg-squid-monitor.cern.ch::awstats-$SITEPROJ/$AWSTATS_UPLOAD_DIR

